"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, {enumerable: true, configurable: true, writable: true, value}) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};

// src/index.ts
var _esbuild = require('esbuild');
var _vitepluginfiles = require('vite-plugin-files'); var _vitepluginfiles2 = _interopRequireDefault(_vitepluginfiles);

// src/options.ts
function resolveOptions(userOptions) {
  const {
    routesDir = [{dir: "src/pages", baseRoute: ""}],
    extensions = ["tsx"],
    exclude = [/components/],
    hash = false
  } = userOptions;
  return Object.assign({}, userOptions, {routesDir, extensions, exclude, hash});
}

// src/utils.ts
function selectProperty(object, filterKey) {
  return Object.keys(object).filter(filterKey).reduce((pre, next) => {
    pre[next] = object[next];
    return pre;
  }, {});
}

// src/generateRoutes.ts
var SPACIALFILEREGEXP = /^(_layout|index|_loading|404)$/;
var HANDDOWNSPACIALFILEREGEXP = /^(_loading|404)$/;
function makePath(parentPath, filename) {
  return `${parentPath}/${filename.replace(/^\$/, ":").replace(/\$$/, "?")}`.replace(/\/\//, "/");
}
function makeFileName(name) {
  return name.substring(0, name.lastIndexOf("."));
}
function makeFilePath(path) {
  if (path[0] !== "/")
    path = "/" + path;
  return path;
}
function generateFile(tree, parentPath, loading) {
  const filename = makeFileName(tree.name);
  const currentPath = makePath(parentPath, filename);
  const route = {path: currentPath, file: makeFilePath(tree.relativePath)};
  if (loading)
    route.loading = makeFilePath(loading.relativePath);
  return route;
}
function generateRoutesRecurse(tree, parentPath, optionRoutes, superSpecialFiles = {}, isRoot = true) {
  let generatedRoutes = [];
  if (!tree.children || tree.children.length <= 0)
    return generatedRoutes;
  const currentPath = makePath(parentPath, isRoot ? "/" : tree.name);
  const filesMap = {};
  const specialFilesMap = __spreadValues(__spreadValues({}, superSpecialFiles), optionRoutes);
  const dirsMap = {};
  tree.children.forEach((el) => {
    if (el.type === "file") {
      const filename = makeFileName(el.name);
      const isSpacialFile = SPACIALFILEREGEXP.test(filename);
      if (isSpacialFile) {
        specialFilesMap[filename] = el;
      } else {
        filesMap[el.name] = el;
      }
    }
    if (el.type === "directory")
      dirsMap[el.name] = el;
  });
  const dirRoutes = Object.keys(dirsMap).map((key) => generateRoutesRecurse(dirsMap[key], currentPath, optionRoutes, selectProperty(specialFilesMap, (key2) => HANDDOWNSPACIALFILEREGEXP.test(key2)), false)).reduce((pre, next) => pre.concat(next), []);
  generatedRoutes = generatedRoutes.concat(dirRoutes);
  const fileRoutes = Object.keys(filesMap).map((key) => generateFile(filesMap[key], currentPath, specialFilesMap["_loading"]));
  generatedRoutes = generatedRoutes.concat(fileRoutes);
  if (specialFilesMap["index"]) {
    const indexRoute = generateFile(specialFilesMap["index"], currentPath, specialFilesMap["_loading"]);
    indexRoute.path = indexRoute.path.replace(/\/index$/, "/");
    indexRoute.exact = true;
    generatedRoutes.push(indexRoute);
  }
  if (specialFilesMap["404"]) {
    const notFoundRoute = generateFile(specialFilesMap["404"], currentPath);
    notFoundRoute.path = `${currentPath}/*`.replace("//", "/");
    notFoundRoute.exact = false;
    generatedRoutes.push(notFoundRoute);
  }
  if (specialFilesMap["_layout"]) {
    generatedRoutes = [
      {
        path: currentPath,
        file: makeFilePath(specialFilesMap["_layout"].relativePath),
        routes: generatedRoutes,
        loading: makeFilePath(specialFilesMap["_loading"].relativePath)
      }
    ];
  }
  return generatedRoutes;
}
var generateRoutes = (trees, options) => {
  const routes = trees.map(([tree, pageDir]) => {
    let {baseRoute} = pageDir;
    return generateRoutesRecurse(tree, baseRoute, generateOptionRoutes(options));
  }).reduce((pre, next) => pre.concat(next), []);
  return routes;
};
var generateOptionRoutes = (options) => {
  const {notFound, loading} = options;
  const optionRoutes = {};
  if (loading) {
    optionRoutes["_loading"] = {name: "_loading", type: "file", path: loading, relativePath: loading};
  }
  if (notFound) {
    optionRoutes["404"] = {name: "404", type: "file", path: notFound, relativePath: notFound};
  }
  return optionRoutes;
};

// src/generateString.ts
var importCache = {};
var generateImport = (path) => {
  if (importCache[path])
    return importCache[path];
  const importVar = path.replace(/\.\w+$/, "").replace(/[\W_]([a-z A-Z])/g, (str) => str.toUpperCase()).replace(/[\W_]/g, "");
  importCache[path] = importVar;
  return importVar;
};
function stringifyRoute(route) {
  const {path, file, exact, routes} = route;
  const routesString = routes ? `routes:[${routes.map((el) => stringifyRoute(el)).join(",\n")}]` : "";
  const exactString = `exact:${routes ? "false" : !!exact},`;
  const loadingString = route.loading ? `, { fallback: <${generateImport(route.loading)} /> }` : "";
  if (/\/\*$/.test(path)) {
    return `
        {
            path: "${path}",
            component: ${generateImport(route.file)},
            ${exactString}
        }
    `;
  }
  return `
        {
            path: "${path}",
            component: loadable(() => import("${file}")${loadingString}),
            ${exactString}
            ${routesString}
        }
    `;
}
var stringifyRoutes = (routes) => {
  importCache = {};
  return {
    stringRoutes: `[${routes.map((el) => stringifyRoute(el)).join(",\n")}]`,
    stringImport: Object.keys(importCache).reduce((pre, next) => {
      return `${pre} 
import ${importCache[next]} from "${next}"`;
    }, "")
  };
};
var assembleCode = (stringRoutes, stringImport, hash) => {
  const createHistoryString = hash ? "createHashHistory" : "createBrowserHistory";
  return `
        import React, { PropsWithoutRef, ComponentType } from "react";
        import { Router, Switch, Route, Link } from "react-router-dom";
        import { ${createHistoryString} } from "history";

        import loadable from "@loadable/component";

        ${stringImport}
        
        export interface RouteConfig {
            path: string;
            exact: boolean;
            component: ComponentType;
            routes?: RouteConfig[];
        }
        
        function RouteWithSubRoutes(route: PropsWithoutRef<RouteConfig>) {
            return (
                <Route
                    exact={route.exact}
                    path={route.path}
                    render={(props) => (
                        <route.component {...props}>{route.routes && renderRoute(route.routes)}</route.component>
                    )}
                />
            );
        }
        
        function renderRoute(routes: RouteConfig[]) {
            return (
                <Switch>
                    {routes.map((route, i) => (
                        <RouteWithSubRoutes key={i} {...route} />
                    ))}
                </Switch>
            );
        }

        export const history = ${createHistoryString}()

        export const routes = ${stringRoutes}
        
        export default function RouterRoot() {
            return <Router history={history}>{renderRoute(routes)}</Router>;
        }
    `;
};

// src/index.ts
function VitePluginReactRouter(userOptions) {
  const options = resolveOptions(userOptions);
  const {routesDir, extensions, exclude} = options;
  const {name, enforce, resolveId, configResolved, configureServer, load} = _vitepluginfiles2.default.call(void 0, {
    name: "vite-plugin-react-router",
    virtualId: "generated-routes",
    filesDir: routesDir,
    extensions,
    exclude,
    onFilterFile: () => {
      return true;
    },
    onGeneratedClient: (trees) => {
      const routes = generateRoutes(trees, options);
      const {stringRoutes, stringImport} = stringifyRoutes(routes);
      const code = assembleCode(stringRoutes, stringImport, options.hash);
      return code;
    }
  });
  let command;
  return {
    name,
    enforce,
    resolveId,
    configResolved(config) {
      command = config.command;
      configResolved(config);
    },
    configureServer(server) {
      options.devServer = server;
      configureServer && configureServer(server);
    },
    load,
    async transform(_code, id) {
      var _a;
      if (id === "/generated-routes") {
        const esbuildResult = command == "serve" ? await ((_a = options.devServer) == null ? void 0 : _a.transformWithEsbuild(_code, "/generated-routes.tsx", {
          loader: "tsx"
        })) : await _esbuild.transform.call(void 0, _code, {loader: "tsx"});
        return esbuildResult == null ? void 0 : esbuildResult.code;
      }
    }
  };
}
var src_default = VitePluginReactRouter;


exports.default = src_default;
